
/*	Copyright (C) 2005, 2006 by Andy Grebe a.k.a. tecknophreak(tecknophreak@gmail.com)
 *
 *	This file is part of TouchMe Games.
 *
 *	TouchMe Games is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	TouchMe Games is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with TouchMe Games; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 

// events for DART_LOWBALL

#include <dartlowball.h>
#include <algorithm>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <termios.h>
#include <unistd.h>

void DART_LOWBALL::exitGame(GtkWidget *widget, GdkEvent *event, gpointer ptr) {
	DART_LOWBALL *g(reinterpret_cast<DART_LOWBALL *>(ptr));
	SCORE_STRUCT newScore;
	
	newScore.name = "";
	newScore.score = g->finalScore[g->winner];
	newScore.date = "";
	
	g->GAME_PTR::addScore(g->scores, newScore);
	
	
	g->GAME_PTR::saveScores(g->scores);
	
	gtk_widget_hide(g->fixed);
	
	g_source_remove(g->timer);
	
	g->dartBoard.closeDart();
}

gboolean DART_LOWBALL::timeout(gpointer ptr) {
	DART_LOWBALL *dart(reinterpret_cast<DART_LOWBALL *>(ptr));
	int score, totScore;
	std::stringstream sstr;
	
	
	if (dart->currentDart <= 2) {
		int a, x(0);
		if (!dart->dartBoard.gotDart()) {
			return TRUE;
		}
		dart->dartBoard.readDart(x, a);
		
		if (a == 2) {
			++dart->doubles[dart->currentPlayer];
		}
		else if (a == 3) {
			++dart->triples[dart->currentPlayer];
		}
		
		if (x > 20) {
			++dart->bulls[dart->currentPlayer];
		}
		
		score = x * a;
		
		// Incoming dart
		totScore = atoi(gtk_label_get_text((GtkLabel *)dart->playerScoreLabels[dart->currentPlayer]));
		totScore += score;
		++dart->darts[dart->currentPlayer];

		sstr << totScore;
		gtk_label_set_text((GtkLabel *)dart->playerScoreLabels[dart->currentPlayer], sstr.str().c_str());
		gtk_label_set_text((GtkLabel *)dart->currentTotalLabel, sstr.str().c_str());
		dart->currentScore += score;
		
		sstr.str("");
		if (a == 1) {
			sstr << "Single ";
			if (x < 25) {
				sstr << score;
			}
			else {
				sstr << "BULL";
			}
		}
		else if (a == 2) {
			sstr << "Double ";
			if (x < 25) {
				sstr << score / a;
			}
			else {
				sstr << "BULL";
			}
		}
		else if (a == 3) {
			sstr << "Triple " << score / a;
			
		}
		gtk_label_set_text((GtkLabel *)dart->dartLabels[dart->currentDart], sstr.str().c_str());
		gtk_widget_show(dart->dartLabels[dart->currentDart]);
		
		sstr.str("");
		sstr << dart->currentScore;
		gtk_label_set_text((GtkLabel *)dart->currentScoreLabel, sstr.str().c_str());
		gtk_widget_hide(dart->dartImages[dart->currentDart++]);
	}
	else {
		if (dart->currentDart == 3) {
			int player;
			float ppd(0.0), points;
			// PPD Above each player's score
			++dart->rounds[dart->currentPlayer];
			player = dart->currentPlayer;
			points = atoi(gtk_label_get_text((GtkLabel *)dart->playerScoreLabels[player]));
			
			if (dart->darts[player]) {
				ppd = points / (float)(dart->darts[player]);
			}
			else {
				ppd = 0.0;
			}
			
			sstr.str("");
			sstr << "Player " << player + 1 << " - " << std::setprecision(3) << ppd;
			gtk_label_set_text((GtkLabel *)dart->playerLabels[player], sstr.str().c_str());


		}
		else if (dart->currentDart == 23) {
			int player;
			float ppd(0.0), points;
			dart->dartBoard.pause();
			// get the next player's PPD going
			if (dart->currentPlayer + 1 == dart->numberOfPlayers) {
				player = 0;
				std::cout << "Round " << dart->round << std::endl;
				if (dart->round == 8) {
					int topScore(1440);
					for (int i = 0; i < dart->numberOfPlayers; ++i) {
						int tmpScore = atoi(gtk_label_get_text((GtkLabel *)dart->playerScoreLabels[i]));
						std::cout << "tmpScore " << tmpScore << std::endl;
						if (tmpScore < topScore) {
							topScore = tmpScore;
							dart->winner = i;
						}
					}
					dart->gameState = GAME_OVER1;
					dart->createGameOver1();
					return FALSE;
				}
			}
			else {
				player = dart->currentPlayer + 1;
			}
			
			points = atoi(gtk_label_get_text((GtkLabel *)dart->playerScoreLabels[player]));
			
			if (dart->darts[player]) {
				ppd = points / (float)(dart->darts[player]);
			}
			else {
				ppd = 0.0;
			}
			std::cout << "Points per dart " << ppd << std::endl;
			std::cout << "points: " << points << std::endl;
			std::cout << "darts: " << dart->darts[player] << std::endl;
			sstr.str("");
			sstr << "Player " << player + 1;
			gtk_label_set_text((GtkLabel *)dart->ppdPlayerLabel, sstr.str().c_str());
			
			sstr.str("");
			sstr << "PPD: " << std::setprecision(3) << ppd;
			
			gtk_label_set_text((GtkLabel *)dart->ppdLabel, sstr.str().c_str());
			
			gtk_widget_show_all(dart->ppdWindow);
			
		}
		++dart->currentDart;
	}
	if (dart->currentDart == 43) {
		dart->currentDart = 0;
		gtk_widget_show(dart->dartImages[0]);
		gtk_widget_show(dart->dartImages[1]);
		gtk_widget_show(dart->dartImages[2]);
		gtk_widget_hide(dart->dartLabels[0]);
		gtk_widget_hide(dart->dartLabels[1]);
		gtk_widget_hide(dart->dartLabels[2]);
		dart->currentScore = 0;
		gtk_label_set_text((GtkLabel *)dart->currentScoreLabel, "0");
		
		if (++dart->currentPlayer == dart->numberOfPlayers) {
			dart->currentPlayer = 0;
			++dart->round;
			sstr.str("");
			sstr << "Round " << std::setw(3) << dart->round;
			gtk_label_set_text((GtkLabel *)dart->roundLabel, sstr.str().c_str());
			
		}
		dart->beginningScore = atoi(gtk_label_get_text((GtkLabel *)dart->playerScoreLabels[dart->currentPlayer]));
		gtk_label_set_text((GtkLabel *)dart->currentTotalLabel,
				    gtk_label_get_text((GtkLabel *)dart->playerScoreLabels[dart->currentPlayer]));
		
		sstr.str("");
		sstr << "Player " << dart->currentPlayer + 1 << " Score";
		gtk_label_set_text((GtkLabel *)dart->playerLabel, sstr.str().c_str());
		gtk_widget_hide_all(dart->ppdWindow);
		dart->dartBoard.unpause();
	}
	return TRUE;
}

void DART_LOWBALL::changeNumPlayers(GtkWidget *widget, gpointer ptr) {
	std::stringstream sstr;
	int i(atoi(gtk_entry_get_text((GtkEntry *)ptr)));
	if (++i > 8) {
		i = 1;
	}
	sstr << i;
	gtk_entry_set_text((GtkEntry *)ptr, sstr.str().c_str());
}

void DART_LOWBALL::onOffToggle(GtkWidget *widget, gpointer ptr) {
	std::string str;
	str = gtk_label_get_text((GtkLabel *)ptr);
	if (str[1] == 'n') {
		gtk_label_set_text((GtkLabel *)ptr, "Off");
	}
	else {
		gtk_label_set_text((GtkLabel *)ptr, "On");
	}
}

void DART_LOWBALL::startGame(GtkWidget *widget, gpointer ptr) {
	DART_LOWBALL *dart((DART_LOWBALL *)ptr);
	dart->doubleBullOn = strcmp(gtk_label_get_text((GtkLabel *)dart->doubleBullLabel), "On") == 0 ? true: false;
	dart->dartBoard.setDoubleBull(dart->doubleBullOn);
	dart->numberOfPlayers = atoi(gtk_entry_get_text((GtkEntry *)dart->numPlayersEntry));
	((DART_LOWBALL *)ptr)->initScreen();
}

